// Diese Klasse beinhaltet ein Individuum.
// Ein Individuum besteht aus 3 Chromosomen:
// 		C1:	ein Chromosom, welches aus den freien Stellen besteht, d.h
//			dort, wo Objekte aufgestellt werden knnen
//		C2: ein Chromosom, welches aus allen Objket_ids besteht
//		C3: ein Chromosom, welches aus den Ausrichtungen besteht,
//			wobei diese binr kodiert sind 00..N; 01...O, 10...S; 11...W

package ga;

import java.util.BitSet;

 
public class Individuum
{
   // Chromosom1 reprsentiert die freien Stellen
   Chromosom_bit c1;
   // Chromosom2 reprsentiert aufgestellten Objekte
   Chromosom_int c2;
   // Chromosom3 reprsentiert die Ausrichtungen der der
   // aufgestellten Objekte
   Chromosom_bit c3;
   // Fitnesswert eines Individuums
   int fitness;
   
   
   // Dieser Konstruktor erzeugt ein neues Inidviduum
   public Individuum(BitSet in_c1, int[] in_c2, BitSet in_c3, int fitness)
   {
      c1 = new Chromosom_bit(in_c1, GA.chromlaenge_C1);
      c2 = new Chromosom_int(in_c2);
      c3 = new Chromosom_bit(in_c3, GA.chromlaenge_C3);
      this.fitness = fitness;
   }

   // Dieser Konstruktor erzeugt ein neues Inidviduum
   public Individuum(Chromosom_bit in_c1, Chromosom_int in_c2, 
                     Chromosom_bit in_c3, int fitness)
   {
      c1 = new Chromosom_bit(in_c1);
      c2 = new Chromosom_int(in_c2);
      c3 = new Chromosom_bit(in_c3);
      this.fitness = fitness;
   }


   // Dieser Konstruktor erzeugt unter Angabe der beiden Eltern
   // ein neues Individuum. Dabei wird ein Crossover durchgefhrt.
   // Die Reihenfolge der beiden Elternteile ist dabei relevant. 
   public Individuum(Individuum parent1, Individuum parent2)
   {      
      int welches = GA.c_punkt(3);
      
      switch (welches) {
      	// beim ersten Chromosom wird ein Crossover durchgefhrt,
      	// bei allen anderen nicht
      	case 0: c1 = new Chromosom_bit(parent1.chrom1(), parent2.chrom1());
                c2 = new Chromosom_int(parent1.chrom2());
                c3 = new Chromosom_bit(parent1.chrom3());
                break;	
        // beim zweiten Chromosom wird ein Crossover durchgefhrt,
      	// bei allen anderen nicht
      	case 1: c1 = new Chromosom_bit(parent1.chrom1());
                c2 = new Chromosom_int(parent1.chrom2());
                c2.crossover();
                c3 = new Chromosom_bit(parent1.chrom3());
                break;	
        // beim dritten Chromosom wird ein Crossover durchgefhrt,
      	// bei allen anderen nicht
      	case 2: c1 = new Chromosom_bit(parent1.chrom1());
                c2 = new Chromosom_int(parent1.chrom2());
                c3 = new Chromosom_bit(parent1.chrom3(), parent2.chrom3());
                break;	
        }
   
            
      
      // Finesswert mu erst ermittelt werden
      fitness = -1;
      
    }


  
   //_________________________________________________________
   // Gibt das Chromosom1 des Individuums zurck	 
   public Chromosom_bit chrom1()
   {
      return c1;
   }

   //_________________________________________________________
   // Gibt das Chromosom2 des Individuums zurck	 
   public Chromosom_int chrom2()
   {
      return c2;
   }

   //_________________________________________________________
   // Gibt das Chromosom3 des Individuums zurck	 
   public Chromosom_bit chrom3()
   {
      return c3;
   }
  
   //_________________________________________________________
   // Gibt den Fitnesswert des Individuums zurck	 
   public int ausgabe_fitness()
   {
      return fitness;
   }

  
   //_________________________________________________________
   // ndert den Fitnesswert des Individuums  
   public void aendere_fitness(int fitness)
   {
      this.fitness = fitness;
   }
  
   
   //_________________________________________________________
   // Mutiert ein Individuum
   // dabei wird nur ein Chromosom mutiert.  
   public void mutiere()
   {
      int welches = GA.c_punkt(3);
      
      //System.out.println(" Mutiere C" + welches);
      
      switch (welches) {
      	case 0: c1.mutiere(); break;	
        case 1: c2.mutiere(); break;
        case 2: c3.mutiere(); break;      
      }
      
      // fitness mu neu berechnet werden
      fitness = -1;
   }
  

}
